@@ -12,7 +12,7 @@ family_factory <- function(default_link,family,variance) {
 }
 
 ## suppress code warnings for nbinom2; can't use .Theta <- NULL trick here ...
-globalVariables(".Theta") 
+utils::globalVariables(".Theta") 
 
 ## attempt to guess whether calling function has been called from glm.fit ...
 in_glm_fit <- function() {
@@ -50,7 +50,7 @@ make_family <- function(x,link) {
 ##'
 ##' 
 ##' @aliases family_glmmTMB
-##' @param link (character) link function for the conditional mean ("log", "logit", "probit", "inverse", "cloglog", or "identity")
+##' @param link (character) link function for the conditional mean ("log", "logit", "probit", "inverse", "cloglog", "identity", or "sqrt")
 ##' @return returns a list with (at least) components
 ##' \item{family}{length-1 character vector giving the family name}
 ##' \item{link}{length-1 character vector specifying the link function}
@@ -59,27 +59,34 @@ make_family <- function(x,link) {
 ##' predicted variance (scaled by \code{sigma(.)})
 ##' }
 ##' @details
-##' If specified, the dispersion model uses a log link. Denoting the dispersion parameter
-##' as phi=exp(eta) (where eta is the linear predictor from the dispersion model)
-##' and the predicted mean as mu:
+##' If specified, the dispersion model uses a log link. Denoting the variance as \eqn{V}, the dispersion parameter
+##' as \eqn{\phi=\exp(\eta)}{phi=exp(eta)} (where \eqn{\eta}{eta} is the linear predictor from the dispersion model), 
+##' and the predicted mean as \eqn{\mu}{mu}:
 ##'  \describe{
-##'      \item{gaussian}{(from base R): constant variance=phi}
-##'      \item{Gamma}{(from base R) phi is the shape parameter, i.e variance=mu*phi}
-##'      \item{nbinom2}{variance increases quadratically with the mean (Hardin & Hilbe 2007),
-##' i.e. variance=mu*(1+mu/phi)}
-##'      \item{nbinom1}{variance increases linearly with the mean (Hardin & Hilbe 2007),
-##' i.e. variance=mu*(1+phi)}
-##'      \item{compois}{is the Conway-Maxwell Poisson parameterized with the exact mean
-##'           which differs from the COMPoissonReg package (Sellers & Lotze 2015)}
-##'      \item{genpois}{is the generalized Poisson distribution}
-##'      \item{beta}{follows the parameterization of Ferrari and Cribari-Neto (2004) and the \code{betareg} package,
-##'     i.e. variance=mu*(1-mu)}
+##'      \item{gaussian}{(from base R): constant \eqn{V=\phi}{V=phi}}
+##'      \item{Gamma}{(from base R) phi is the shape parameter. \eqn{V=\mu\phi}{V=mu*phi}}
+##'       \item{ziGamma}{a modified version of \code{Gamma} that skips checks for zero values, allowing it to be used to fit hurdle-Gamma models}
+##'      \item{nbinom2}{Negative binomial distribution: quadratic parameterization (Hardin & Hilbe 2007). \eqn{V=\mu(1+\mu/\phi) = \mu+\mu^2/\phi}{V=mu*(1+mu/phi) = mu+mu^2/phi}.}
+##'      \item{nbinom1}{Negative binomial distribution: linear parameterization (Hardin & Hilbe 2007). \eqn{V=\mu(1+\phi)}{V=mu*(1+phi)}}
+##'      \item{truncated_nbinom2}{Zero-truncated version of nbinom2: variance expression from Shonkwiler 2016. Simulation code (for this and the other truncated count distributions) is taken from C. Geyer's functions in the \code{aster} package; the algorithms are described in \href{https://cran.r-project.org/package=aster/vignettes/trunc.pdf}{this vignette}.}
+##'      \item{compois}{Conway-Maxwell Poisson distribution: parameterized with the exact mean (Huang 2017), which differs from the parameterization used in the \pkg{COMPoissonReg} package (Sellers & Shmueli 2010, Sellers & Lotze 2015). \eqn{V=\mu\phi}{V=mu*phi}.}
+##'      \item{genpois}{Generalized Poisson distribution (Consul & Famoye 1992). \eqn{V=\mu\exp(\eta)}{V=mu*exp(eta)}. (Note that Consul & Famoye (1992) define \eqn{\phi}{phi} differently.) Our implementation is taken from the \code{HMMpa} package, based on Joe and Zhu (2005) and implemented by Vitali Witowski.}
+##'      \item{beta}{Beta distribution: parameterization of Ferrari and Cribari-Neto (2004)
+##' and the \pkg{betareg} package (Cribari-Neto and Zeileis 2010); \eqn{V=\mu(1-\mu)/(\phi+1)}{V=mu*(1-mu)/(phi+1)}}
+##'     \item{betabinomial}{Beta-binomial distribution: parameterized according to Morris (1997). \eqn{V=\mu(1-\mu)(n(\phi+n)/(\phi+1))}{V=mu*(1-mu)*(n*(phi+n)/(phi+1))}}
+##'      \item{tweedie}{Tweedie distribution: \eqn{V=\phi\mu^p}{V=phi*mu^p}. The power parameter is restricted to the interval \eqn{1<p<2}. Code taken from the \code{tweedie} package, written by Peter Dunn.}
 ##' }
 ##' @references
 ##' \itemize{
+##' \item Consul PC & Famoye F (1992). "Generalized Poisson regression model." Communications in Statistics: Theory and Methods 21:89–109.
 ##' \item Ferrari SLP, Cribari-Neto F (2004). "Beta Regression for Modelling Rates and Proportions." \emph{J. Appl. Stat.}  31(7), 799-815.
 ##' \item Hardin JW & Hilbe JM (2007). "Generalized linear models and extensions." Stata Press.
+##' \item Huang A (2017). "Mean-parametrized Conway–Maxwell–Poisson regression models for dispersed counts." \emph{Statistical Modelling} 17(6), 1-22.
+##' \item Joe H, Zhu R (2005). "Generalized Poisson Distribution: The Property of Mixture of Poisson and Comparison with Negative Binomial Distribution." \emph{Biometrical Journal} 47(2): 219–29. doi:10.1002/bimj.200410102.
+##' \item Morris  W (1997). "Disentangling Effects of Induced Plant Defenses and Food Quantity on Herbivores by Fitting Nonlinear Models." \emph{American Naturalist} 150:299-327.
 ##' \item Sellers K & Lotze T (2015). "COMPoissonReg: Conway-Maxwell Poisson (COM-Poisson) Regression". R package version 0.3.5. https://CRAN.R-project.org/package=COMPoissonReg
+##' \item Sellers K & Shmueli G (2010) "A Flexible Regression Model for Count Data." \emph{Annals of Applied Statistics} 4(2), 943–61. https://doi.org/10.1214/09-AOAS306.
+##' \item Shonkwiler, J. S. (2016). "Variance of the truncated negative binomial distribution." \emph{Journal of Econometrics} 195(2), 209–210. doi:10.1016/j.jeconom.2016.09.002
 ##' }
 ##' @export
 ##' @importFrom stats make.link
@@ -190,12 +197,28 @@ truncated_poisson <- function(link="log") {
 }
 
 #' @rdname nbinom2
+#' @importFrom stats dnbinom pnbinom
 #' @export	       	
 truncated_nbinom2 <- function(link="log") {
+    theta_errstr <- "theta (nbinom parameter) neither passed as an argument nor stored in enviroment"
+    missing_theta <- "one" ## or "stop" or "na"
     r <- list(family="truncated_nbinom2",
-           variance=function(mu,theta) {
-               stop("variance for truncated nbinom2 family not yet implemented")
-         })
+              variance=function(mu,theta) {
+                      if (missing(theta)) {
+                          if (!exists(".Theta")) {
+                              theta <- switch(missing_theta, one = 1, na = NA_real_, 
+                                              stop = stop(theta_errstr))
+                          }
+                          else {
+                              theta <- .Theta
+                          }
+                      }
+                      a <- theta
+                      c <- 0 ## truncation point
+                      mu_star <- mu + (a+mu)*(c+1)*dnbinom(c+1,mu=mu,size=theta)/
+                          (a*(1-pnbinom(c,mu=mu,size=theta)))
+                      return(mu_star + c*(mu_star-mu) +mu_star*mu*(1+1/a)-mu_star^2)
+              })
     return(make_family(r,link))
 }
 
@@ -221,8 +244,14 @@ beta_family <- function(link="logit") {
     r <- list(family="beta",
               variance=function(mu) { mu*(1-mu) },
               initialize=expression({
-                  if (any(y <= 0 | y >= 1)) 
-                      stop("y values must be 0 < y < 1")
+                  if (exists("ziformula") && !ident(ziformula, ~0)) {
+                      if (any(y < 0 | y >= 1)) {
+                          stop("y values must be 0 <= y < 1")
+                      }
+                  } else {
+                      if (any(y <= 0 | y >= 1)) 
+                          stop("y values must be 0 < y < 1")
+                  }
                   mustart <- y
               }))
     return(make_family(r,link))
@@ -232,12 +261,20 @@ beta_family <- function(link="logit") {
 
 #' @rdname nbinom2
 #' @export
+## variance= (Wikipedia)
+## n*alpha*beta*( alpha + beta + n )/ ((alpha+beta)^2*(alpha+beta+1))
+## alpha = p*theta
+## beta = (1-p)*theta
+## -> n*p*(1-p)*theta^2*(theta+n)/(theta^2*(theta+1))
+## =  n*p*(1-p)*(theta+n)/(theta+1)
+##  *scaled* variance (dependence on mu only) is still just mu*(1-mu);
+##  scaling is n*(theta+n)/(theta+1) (vs. simply n for the binomial)
 betabinomial <- function(link="logit") {
     r <- list(family="betabinomial",
-              variance=function(mu,phi) {
-        stop("variance for betabinomial family not yet implemented")
-    },
-    initialize = binomial()$initialize)
+              variance = function(mu, phi) {
+                  mu*(1-mu)
+              },
+              initialize = binomial()$initialize)
     return(make_family(r,link))
 }
 
@@ -297,3 +334,26 @@ getCapabilities <- function(what="all",check=FALSE) {
         return(family_OK)
     }
 }
+
+#' @export
+#' @rdname nbinom2
+ziGamma <- function(link="inverse") {
+    g <- stats::Gamma(link=link)
+    ## stats::Gamma does clever deparsing stuff ... need to work around it ...
+    if (is.function(link)) {
+        g$link <- deparse(substitute(link))
+    } else g$link <- link
+    ## modify initialization to allow zero values in zero-inflated cases
+    g$initialize <- expression({
+        if (exists("ziformula") && !ident(ziformula, ~0)) {
+            if (any(y < 0)) stop("negative values not allowed for the 'Gamma' family with zero-inflation")
+            } else {
+                if (any(y <= 0)) stop("non-positive values not allowed for the 'Gamma' family")
+            }
+            n <- rep.int(1, nobs)
+            mustart <- y
+    })
+
+   return(g)
+}
+
