@@ -60,7 +60,7 @@ startParams <- function(parameters,
 
     # get either dunn-smyth residuals or
     fam <- family$family
-    res.families <- c("poisson", "nbinom2", "binomial", "guassian")
+    res.families <- c("poisson", "nbinom2", "binomial", "gaussian")
     if (fam %in% res.families) {
       #### Get the dunn smyth residuals
       if (fam == "poisson") {
@@ -116,7 +116,7 @@ startParams <- function(parameters,
     names(par.list) <- c("theta", "b", "fact_load")
     # Use glmmTMB to get initial starting values for factor loadings and latent variables
     fr.res <- cbind(fr, resid)
-    ranForm <- findbars(RHSForm(formula))
+    ranForm <- no_specials(findbars_x(RHSForm(formula)))
     nrr <- length(namBlk)
     rrTrm <- lapply(1:length(namBlk), function(x) as.character(ranForm[ranForm == namBlk][[x]]))
     x <- sapply(1:nrr, function(x) paste(rrTrm[[x]][2], rrTrm[[x]][1], rrTrm[[x]][3]))
@@ -380,9 +380,10 @@ mkTMBStruc <- function(formula, ziformula, dispformula,
     ##  (which confuse MakeADFun)
     yobs = c(yobs),
     respCol,
-    offset = condList$offset,
-    zioffset = ziList$offset,
-    doffset = dispList$offset,
+    ## strip attributes from offset terms
+    offset = c(condList$offset),
+    zioffset = c(ziList$offset),
+    doffset = c(dispList$offset),
     weights,
     size = c(size),
     ## information about random effects structure
@@ -523,9 +524,9 @@ getXReTrms <- function(formula, mf, fr, ranOK=TRUE, type="", contrasts, sparse=F
         mf$formula <- tt
         terms_fixed <- terms(eval(mf,envir=environment(fixedform)))
         if (!sparse) {
-            X <- model.matrix(drop.special2(fixedform), fr, contrasts)
+            X <- model.matrix(drop.special(fixedform), fr, contrasts)
         } else {
-            X <- Matrix::sparse.model.matrix(drop.special2(fixedform), fr, contrasts)
+            X <- Matrix::sparse.model.matrix(drop.special(fixedform), fr, contrasts)
             ## FIXME? ?sparse.model.matrix recommends MatrixModels::model.Matrix(*,sparse=TRUE)
             ##  (but we may not need it, and would add another dependency etc.)
         }
@@ -551,7 +552,7 @@ getXReTrms <- function(formula, mf, fr, ranOK=TRUE, type="", contrasts, sparse=F
     ## important to COPY formula (and its environment)?
     ranform <- formula
 
-    if (is.null(findbars(ranform))) {
+    if (is.null(findbars_x(ranform))) {
         reTrms <- reXterms <- NULL
         Z <- new("dgCMatrix",Dim=c(as.integer(nobs),0L)) ## matrix(0, ncol=0, nrow=nobs)
         aa <- integer(0) #added for rr to get rank
@@ -563,7 +564,8 @@ getXReTrms <- function(formula, mf, fr, ranOK=TRUE, type="", contrasts, sparse=F
         RHSForm(ranform) <- subbars(RHSForm(reOnly(formula)))
 
         mf$formula <- ranform
-        reTrms <- mkReTrms(findbars(RHSForm(formula)), fr, reorder.terms=FALSE)
+        reTrms <- mkReTrms(no_specials(findbars_x(formula)),
+                           fr, reorder.terms=FALSE)
 
         ss <- splitForm(formula)
         # FIX ME: migrate this (or something like it) down to reTrms,
@@ -787,14 +789,22 @@ binomialType <- function(x) {
 ##'
 ##' Fit a generalized linear mixed model (GLMM) using Template Model Builder (TMB).
 ##' @param formula combined fixed and random effects formula, following lme4 syntax.
-##' @param data data frame containing model variables. Not required, but strongly recommended: see 
+##' @param data data frame (tibbles are OK) containing model variables. Not required, but strongly recommended; if \code{data} is not specified, downstream methods such as prediction with new data (\code{predict(fitted_model, newdata = ...)}) will fail. If it is necessary to call \code{glmmTMB} with model variables taken from the environment rather than from a data frame, specifying \code{data=NULL} will suppress the warning message.
 ##' @param family a family function, a character string naming a family function, or the result of a call to a family function (variance/link function) information. See \code{\link{family}} for a generic discussion of families or \code{\link{family_glmmTMB}} for details of \code{glmmTMB}-specific families.
 ##' @param ziformula a \emph{one-sided} (i.e., no response variable) formula for zero-inflation combining fixed and random effects: the default \code{~0} specifies no zero-inflation. Specifying \code{~.} sets the zero-inflation formula identical to the right-hand side of \code{formula} (i.e., the conditional effects formula); terms can also be added or subtracted. \strong{When using \code{~.} as the zero-inflation formula in models where the conditional effects formula contains an offset term, the offset term will automatically be dropped}. The zero-inflation model uses a logit link.
 ##' @param dispformula a \emph{one-sided} formula for dispersion containing only fixed effects: the default \code{~1} specifies the standard dispersion given any family. The argument is ignored for families that do not have a dispersion parameter. For an explanation of the dispersion parameter for each family, see \code{\link{sigma}}. The dispersion model uses a log link. In Gaussian mixed models, \code{dispformula=~0} fixes the residual variance to be 0 (actually a small non-zero value), forcing variance into the random effects. The precise value can be controlled via \code{control=glmmTMBControl(zero_dispval=...)}; the default value is \code{sqrt(.Machine$double.eps)}.
 ##' @param weights weights, as in \code{glm}. Not automatically scaled to have sum 1.
 ##' @param offset offset for conditional model (only).
 ##' @param contrasts an optional list, e.g., \code{list(fac1="contr.sum")}. See the \code{contrasts.arg} of \code{\link{model.matrix.default}}.
-##' @param na.action how to handle missing values, see \code{\link{na.action}} and \code{\link{model.frame}}. From \code{\link{lm}}: \dQuote{The default is set by the \code{\link{na.action}} setting of \code{\link{options}}, and is \code{\link{na.fail}} if that is unset. The \sQuote{factory-fresh} default is \code{\link{na.omit}}.}
+##' @param na.action a function that specifies how to handle observations
+##' containing \code{NA}s.  The default action (\code{na.omit},
+##' inherited from the 'factory fresh' value of
+##' \code{getOption("na.action")}) strips any observations with any
+##' missing values in any variables. Using \code{na.action = na.exclude}
+##' will similarly drop observations with missing values while fitting the model,
+##' but will fill in \code{NA} values for the predicted and residual
+##' values for cases that were excluded during the fitting process
+##' because of missingness.
 ##' @param se whether to return standard errors.
 ##' @param verbose whether progress indication should be printed to the console.
 ##' @param doFit whether to fit the full model, or (if FALSE) return the preprocessed data and parameter objects, without fitting the model.
@@ -824,7 +834,7 @@ binomialType <- function(x) {
 ##' \item \code{mat} (* Matérn process correlation)
 ##' \item \code{toep} (* Toeplitz)
 ##' }
-##' Structures marked with * are experimental/untested. See \code{vignette("covstruct", package = "glmmTMB")} for more information.
+##' Structures marked with * are experimental/untested. See \code{vignette("covstruct", package = "glmmTMB")}< for more information.
 ##' \item For backward compatibility, the \code{family} argument can also be specified as a list comprising the name of the distribution and the link function (e.g. \code{list(family="binomial", link="logit")}). However, \strong{this alternative is now deprecated}; it produces a warning and will be removed at some point in the future. Furthermore, certain capabilities such as Pearson residuals or predictions on the data scale will only be possible if components such as \code{variance} and \code{linkfun} are present, see \code{\link{family}}.
 ##' }
 ##'
@@ -895,7 +905,7 @@ glmmTMB <- function(
     weights=NULL,
     offset=NULL,
     contrasts=NULL,
-    na.action=na.fail,
+    na.action,
     se=TRUE,
     verbose=FALSE,
     doFit=TRUE,
@@ -922,16 +932,19 @@ glmmTMB <- function(
         }
         family <- get(family, mode = "function", envir = parent.frame())
     }
+
     if (is.function(family)) {
         ## call family with no arguments
         family <- family()
     }
+
     ## FIXME: what is this doing? call to a function that's not really
     ##  a family creation function?
     if (is.null(family$family)) {
       print(family)
       stop("'family' not recognized")
     }
+
     fnames <- names(family)
     if (!all(c("family","link") %in% fnames))
         stop("'family' must contain at least 'family' and 'link' components")
@@ -942,6 +955,14 @@ glmmTMB <- function(
     if (grepl("^quasi", family$family))
         stop('"quasi" families cannot be used in glmmTMB')
 
+    if (inForm(formula, quote(`$`))) {
+        warning("use of the ", sQuote("$"), " operator in formulas is not recommended")
+    }
+
+    if (missing(data)) {
+        warning("use of the ", sQuote("data"), " argument is recommended")
+    }
+
     ## extract family and link information from family object
     link <- family$link
 
@@ -990,7 +1011,7 @@ glmmTMB <- function(
     ## replace . in ziformula with conditional formula, ignoring offset
     if (inForm(ziformula,quote(.))) {
         ziformula <-
-            update(RHSForm(drop.special2(formula),as.form=TRUE),
+            update(RHSForm(drop.special(formula),as.form=TRUE),
                    ziformula)
     }
 
@@ -1343,19 +1364,19 @@ fitTMB <- function(TMBStruc) {
         max.newton.steps <- 5
         newton.tol <- 1e-10
         if (sdr$pdHess) {
-            ## pdHess can be FALSE
-            ##  * Happens for boundary fits (e.g. dispersion close to 0 - see 'spline' example)
-            ##    * Option 1: Fall back to old method
-            ##    * Option 2: Skip Newton iterations
-            for (iter in seq_len(max.newton.steps)) {
-                g <- as.numeric( obj$gr(par) )
-                if (any(is.na(g)) || max(abs(g)) < newton.tol) break
-                par <- par - solve(h, g)
-            }
-        }
-        if (any(is.na(g))) {
+          ## pdHess can be FALSE (FIXME: neither of these fallback options is implemented?)
+          ##  * Happens for boundary fits (e.g. dispersion close to 0 - see 'spline' example)
+          ##    * Option 1: Fall back to old method
+          ##    * Option 2: Skip Newton iterations
+          for (iter in seq_len(max.newton.steps)) {
+            g <- as.numeric( obj$gr(par) )
+            if (any(is.na(g)) || max(abs(g)) < newton.tol) break
+            par <- par - solve(h, g)
+          }
+          if (any(is.na(g))) {
             warning("a Newton step failed in profiling")
             par <- oldpar
+          }
         }
         fit$par <- par
         fit$objective <- obj$fn(par)
