@@ -17,9 +17,9 @@
 ##' fm1 <- glmmTMB(Reaction ~ Days, sleepstudy)
 ##' (f1 <- fixef(fm1))
 ##' f1$cond
-##' ## show full coefficients, including dispersion parameter
-##' unlist(f1)
-##' print.default(f1)
+##' ## show full coefficients, including empty z-i model and
+##' ## constant dispersion parameter
+##' print(f1, print_trivials = TRUE)
 ##' @importFrom nlme fixef
 ##' @export fixef
 ##' @export
@@ -51,11 +51,15 @@ formComp <- function(object,type="dispformula",target) {
 ## dispersion model without a (... ?)
 
 trivialDisp <- function(object) {
-    formComp(object,"dispformula",~1)
+    formComp(object, "dispformula", ~1)
 }
 
 zeroDisp <- function(object) {
-    formComp(object,"dispformula",~0)
+    formComp(object, "dispformula", ~0)
+}
+
+noZI <- function(object) {
+  formComp(object, "ziformula", ~0)
 }
 
 ## no roxygen for now ...
@@ -69,10 +73,10 @@ trivialFixef <- function(xnm,nm) {
 
 ##' @method print fixef.glmmTMB
 ##' @export
-print.fixef.glmmTMB <- function(x, digits = max(3, getOption("digits") - 3), ...)
+print.fixef.glmmTMB <- function(x, digits = max(3, getOption("digits") - 3), print_trivials = FALSE, ...)
 {
   for(nm in names(x)) {
-      if (!trivialFixef(names(x[[nm]]),nm)) {
+      if (print_trivials || !trivialFixef(names(x[[nm]]),nm)) {
           cat(sprintf("\n%s:\n", cNames[[nm]]))
           print.default(format(x[[nm]], digits=digits), print.gap = 2L, quote = FALSE)
       }
@@ -341,8 +345,7 @@ df.residual.glmmTMB <- function(object, ...) {
 ##' @importFrom stats vcov
 ##' @export
 vcov.glmmTMB <- function(object, full=FALSE, include_mapped=FALSE, ...) {
-  ## don't check_dots, car::Anova tries to pass 'complete'
-  ## check_dots(...)
+  check_dots(..., .ignore = "complete")
   REML <- isREML(object)
   if(is.null(sdr <- object$sdr)) {
     warning("Calculating sdreport. Use se=TRUE in glmmTMB to avoid repetitive calculation of sdreport")
@@ -387,12 +390,12 @@ vcov.glmmTMB <- function(object, full=FALSE, include_mapped=FALSE, ...) {
       ##    res <- covF[-nrow(covF),-nrow(covF)]
 
       reNames <- function(tag) {
-          re <- object$modelInfo$reStruc[[paste0(tag,"ReStruc")]]
-          nn <- mapply(function(n,L) paste(n,seq(L),sep="."),
-                 names(re),
-                 sapply(re,"[[","blockNumTheta"))
-          if (length(nn)==0) return(nn)
-          return(paste("theta",gsub(" ","",nn),sep="_"))
+        re <- object$modelInfo$reStruc[[paste0(tag,"ReStruc")]]
+        num_theta <- vapply(re,"[[","blockNumTheta", FUN.VALUE = numeric(1))
+        nn <- mapply(function(n,L) paste(n, seq(L), sep="."),
+                     names(re), num_theta)
+        if (length(nn) == 0) return(nn)
+        return(paste("theta",gsub(" ", "", unlist(nn)), sep="_"))
       }
       ## nameList for estimated variables;
       nameList <- c(nameList,list(theta=reNames("cond"),thetazi=reNames("zi")))
@@ -631,10 +634,10 @@ model.frame.glmmTMB <- function(formula, ...) {
 ##' @param \dots ignored, for method compatibility
 ##' @importFrom stats fitted model.response residuals
 ##' @export
-residuals.glmmTMB <- function(object, type=c("response", "pearson"), ...) {
+residuals.glmmTMB <- function(object, type=c("response", "pearson", "working"), ...) {
     type <- match.arg(type)
-    if(type=="pearson" &((object$call$ziformula != ~0)|(object$call$dispformula != ~1))) {
-        stop("pearson residuals are not implemented for models with zero-inflation or variable dispersion")
+    if (type == "pearson" && (!noZI(object) || !trivialDisp(object))) {
+      stop("pearson residuals are not implemented for models with zero-inflation or variable dispersion")
     }
     na.act <- attr(object$frame,"na.action")
     mr <- napredict(na.act,model.response(object$frame))
@@ -653,14 +656,19 @@ residuals.glmmTMB <- function(object, type=c("response", "pearson"), ...) {
     r <- mr - fitted(object)
     res <- switch(type,
            response=r,
-           pearson={
+           working = {
+               mu.eta <- family(object)$mu.eta
+               p <- predict(object, type = "link", fast = TRUE)
+               r/mu.eta(p)
+           },
+           pearson = {
                if (is.null(v <- family(object)$variance))
                    stop("variance function undefined for family ",
                         sQuote(family(object)$family),"; cannot compute",
                         " Pearson residuals")
                vv <- switch(length(formals(v)),
                             v(fitted(object)),
-                            v(fitted(object),sigma(object)),
+                            v(fitted(object), sigma(object)),
                             stop("variance function should take 1 or 2 arguments"))
                r <- r/sqrt(vv)
                if (!is.null(wts)) {
@@ -1247,7 +1255,7 @@ model.matrix.glmmTMB <- function (object, component="cond", part="fixed", ...)
 ##' @rdname ranef.glmmTMB
 ##' @param x a \code{ranef.glmmTMB} object (i.e., the result of running \code{ranef} on a fitted \code{glmmTMB} model)
 as.data.frame.ranef.glmmTMB <- function(x, ...) {
-    check_dots(...)
+    check_dots(..., .ignore = "stringsAsFactors")
     tmpf <- function(x) do.call(rbind,lapply(names(x),asDf0,x=x,id=TRUE))
     x0 <- lapply(x,tmpf)
     x1 <- Map(function(x,n) {
